'use strict';

module.exports = {
	abs: 'http://ecma-international.org/ecma-262/5.1/#sec-5.2',
	CheckObjectCoercible: 'https://ecma-international.org/ecma-262/5.1/#sec-9.10',
	floor: 'http://ecma-international.org/ecma-262/5.1/#sec-5.2',
	FromPropertyDescriptor: 'https://ecma-international.org/ecma-262/5.1/#sec-8.10.4',
	IsAccessorDescriptor: 'https://ecma-international.org/ecma-262/5.1/#sec-8.10.1',
	IsCallable: 'https://ecma-international.org/ecma-262/5.1/#sec-9.11',
	IsDataDescriptor: 'https://ecma-international.org/ecma-262/5.1/#sec-8.10.2',
	IsGenericDescriptor: 'https://ecma-international.org/ecma-262/5.1/#sec-8.10.3',
	IsPropertyDescriptor: 'https://ecma-international.org/ecma-262/5.1/#sec-8.10',
	SameValue: 'https://ecma-international.org/ecma-262/5.1/#sec-9.12',
	ToBoolean: 'https://ecma-international.org/ecma-262/5.1/#sec-9.2',
	ToInt32: 'https://ecma-international.org/ecma-262/5.1/#sec-9.5',
	ToInteger: 'https://ecma-international.org/ecma-262/5.1/#sec-9.4',
	ToNumber: 'https://ecma-international.org/ecma-262/5.1/#sec-9.3',
	ToObject: 'https://ecma-international.org/ecma-262/5.1/#sec-9.9',
	ToPrimitive: 'https://ecma-international.org/ecma-262/5.1/#sec-9.1',
	ToPropertyDescriptor: 'https://ecma-international.org/ecma-262/5.1/#sec-8.10.5',
	ToString: 'https://ecma-international.org/ecma-262/5.1/#sec-9.8',
	ToUint16: 'https://ecma-international.org/ecma-262/5.1/#sec-9.7',
	ToUint32: 'https://ecma-international.org/ecma-262/5.1/#sec-9.6',
	Type: 'https://ecma-international.org/ecma-262/5.1/#sec-8'
};
